package sun.management.snmp.util;

import com.sun.jmx.mbeanserver.Util;
import java.util.Map;
import java.util.List;
import com.sun.jmx.snmp.SnmpOid;
import java.util.TreeMap;

public abstract class SnmpNamedListTableCache extends SnmpListTableCache
{
    protected TreeMap<String, SnmpOid> names;
    protected long last;
    boolean wrapped;
    static final MibLogger log;
    
    public SnmpNamedListTableCache() {
        this.names = new TreeMap();
        this.last = 0L;
        this.wrapped = false;
    }
    
    protected abstract String getKey(Object p0, List<?> p1, int p2, Object p3);
    
    protected SnmpOid makeIndex(Object o, List<?> list, int n, Object o2) {
        long last = this.last + 1L;
        this.last = last;
        if (last > 4294967295L) {
            SnmpNamedListTableCache.log.debug("makeIndex", "Index wrapping...");
            this.last = 0L;
            this.wrapped = true;
        }
        if (!this.wrapped) {
            return new SnmpOid(this.last);
        }
        int n2 = 1;
        while ((long)n2 < 4294967295L) {
            long last2 = this.last + 1L;
            this.last = last2;
            if (last2 > 4294967295L) {
                this.last = 1L;
            }
            SnmpOid value = new SnmpOid(this.last);
            if (this.names == null) {
                return value;
            }
            if (!this.names.containsValue(value)) {
                if (o == null) {
                    return value;
                }
                if (!((Map)o).containsValue(value)) {
                    return value;
                }
            }
            ++n2;
        }
        return null;
    }
    
    @Override
    protected SnmpOid getIndex(Object x, List<?> list, int n, Object o) {
        String key = this.getKey(x, list, n, o);
        Object o2 = (this.names == null || key == null) ? null : this.names.get(key);
        SnmpOid obj = (o2 != null) ? ((SnmpOid)o2) : this.makeIndex(x, list, n, o);
        if (x != null && key != null && obj != null) {
            ((Map)Util.cast(x)).put(key, obj);
        }
        SnmpNamedListTableCache.log.debug("getIndex", "key=" + key + ", index=" + obj);
        return obj;
    }
    
    @Override
    protected SnmpCachedData updateCachedDatas(Object o, List<?> list) {
        TreeMap<Object, Object> names = new TreeMap();
        SnmpCachedData updateCachedDatas = super.updateCachedDatas(o, list);
        this.names = names;
        return updateCachedDatas;
    }
    
    protected abstract List<?> loadRawDatas(Map<Object, Object> p0);
    
    protected abstract String getRawDatasKey();
    
    protected List<?> getRawDatas(Map<Object, Object> map, String s) {
        List<Object> loadRawDatas = null;
        if (map != null) {
            loadRawDatas = (List)map.get(s);
        }
        if (loadRawDatas == null) {
            loadRawDatas = this.loadRawDatas(map);
            if (loadRawDatas != null && map != null) {
                map.put(s, loadRawDatas);
            }
        }
        return loadRawDatas;
    }
    
    @Override
    protected SnmpCachedData updateCachedDatas(Object x) {
        List<Object> rawDatas = this.getRawDatas((x instanceof Map) ? ((Map)Util.cast(x)) : null, this.getRawDatasKey());
        SnmpNamedListTableCache.log.debug("updateCachedDatas", "rawDatas.size()=" + ((rawDatas == null) ? "<no data>" : ("" + rawDatas.size())));
        TreeMap names = new TreeMap();
        SnmpCachedData updateCachedDatas = super.updateCachedDatas(names, rawDatas);
        this.names = names;
        return updateCachedDatas;
    }
    
    static {
        log = new MibLogger(SnmpNamedListTableCache.class);
    }
}
